/* This file is part of Chunky Monkey IRCD 
 *
 * Chunky Monkey IRCD is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Chunky Monkey IRCD is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Chunky Monkey IRCD; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * $Id: logger.c,v 1.7.4.8 2002/05/19 14:17:41 lloydy Exp $
 *
 */
#include "logger.h"
#include "config_file.h"

static int curr_loglevel = 0;
static int log_fd = -1;

/*
* Open the log file
*/
int open_log(int loglevel)
{
	char log_file[100];

	sprintf(log_file, "%s/%s", VARDIR, "log.ircd");

	log_fd =
	    open(log_file, O_RDWR | O_CREAT | O_APPEND | O_NOFOLLOW,
		 S_IRUSR | S_IWUSR | S_IRGRP);

	if (log_fd == -1) {
		fprintf(stderr, "Unable to open log file %s\n", log_file);
		exit(1);
	}

	/* Set log level */
	curr_loglevel = loglevel;

	return 1;
}


/*
* Close the log file
*/
int close_log()
{
	return close(log_fd);
}

/*
* Actually Log a message to the log file
* XXX - Does multi-level logging work ?
*/
int log_msg(int loglevel, char *format, ...)
{
	char output[1000];
	char times[40];
	time_t curr_time;
	struct tm *time_st;
	va_list list;

	/* Check to see if we should log it */
	if (loglevel > curr_loglevel)
		return 0;

	va_start(list, format);

	curr_time = time(NULL);
	time_st = localtime(&curr_time);

	/* Print the time to the file */
	strftime(times, 1000, "%d/%m/%Y %H:%M:%S - ", time_st);
	
	if(server_config.pid) {
		ircd_sprintf(output, 1000,"%s%d - ",times,server_config.pid);
	} else {
		ircd_sprintf(output, 1000,"%s???? - ",times);
	} 
	write(log_fd, output, strlen(output));
	
	/* Print to screen if needed */
	if (server_config.foreground)
		fprintf(stderr, "%s", output);

	/* Log the message to the file */
	vsnprintf(output, 1000, format, list);
	write(log_fd, output, strlen(output));
	
	if (server_config.foreground)
		fprintf(stderr, "%s\n", output);

	/* Print a newline */
	write(log_fd, "\n", 1);


	va_end(list);

	return 1;
}
