/* This file is part of Chunky Monkey IRCD 
 *
 * Chunky Monkey IRCD is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Chunky Monkey IRCD is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Chunky Monkey IRCD; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * $Id: cmd_privmsg.c,v 1.20.4.15 2002/05/24 21:50:23 lloydy Exp $
 *
 */
#include "modules.h"

/* Export version info */
CMIRCD_MODULE(1.02, CMD_CLIENT | CMD_SERVER);

static int privmsg_to_channel(ServerInfo, ClientInfo, ChannelInfo, char *);
static int privmsg_to_user(ClientInfo, ClientInfo, char *);
static void privmsg_send_to_channel(ChannelInfo, ClientInfo, ServerInfo,
				    char *);

/* 
 * cmd_privmsg - deals with privmsg system 
 */
int cmd_privmsg(ClientInfo c_ptr, ServerInfo s_ptr, Cmd cmd)
{
	ClientInfo recip = NULL;
	ChannelInfo chan = NULL;
	ClientInfo sender = NULL;
	char buffer[BUF_SIZE];
	int userhost = 0,i;

	/* find out if it's a client or a room */
	if (!cmd->params[0] || !cmd->text) {
		ircd_sprintf(buffer, BUF_SIZE,
			     "%s : Not enough parameters\n", cmd->command);
		send_numeric(c_ptr, get_my_server(), 461, buffer);

		return 0;
	}

	/* Try and get real origin */
	if (!c_ptr) {
		sender = client_locate(cmd->origin);

		if (!sender) {
			return 0;
		}
	} else {
		sender = c_ptr;
	}

	/* Try and match channel first */
	chan = locate_channel(cmd->params[0]);
	if (chan) {
		/* Message to channel */
		privmsg_to_channel(s_ptr, sender, chan, cmd->text);
		sender->idle_timer = time(NULL);
		return 1;
	}

	/* Try and match a  user */
	recip = client_locate(cmd->params[0]);
	if (recip) {
		/* Message to client */
		privmsg_to_user(sender, recip, cmd->text);
		sender->idle_timer = time(NULL);
		return 1;
	}

	/* Try for user@server format */
	for(i = 0; cmd->params[0][i]!='\0' && !userhost; i++) {
		if(cmd->params[0][i]=='@') {
			userhost = i;
		}
	}
	if (userhost) {
		ServerInfo s_ptr = NULL;
		
		cmd->params[0][userhost]='\0';
		recip = client_locate(cmd->params[0]);
		s_ptr = server_locate( &(cmd->params[0][userhost+1]) );

		if(recip && recip->server == s_ptr) {
			privmsg_to_user(sender,recip,cmd->text);
			sender->idle_timer = time(NULL);
			return 1;
		}
		
	}

	/* Attempt to match by a server mask */
	if ((cmd->params[0][0] == '$') && (sender->mode & MODE_IRCOP)) {
		size_t max_len = 0;
		char *next_ptr = NULL;
		char trim_buffer[BUF_SIZE];

		max_len = BUF_SIZE - 14 - strlen(recip->nick);
		next_ptr = cmd->text;

		while (next_ptr) {
			if (strlen(next_ptr) > max_len) {
				strncpy(trim_buffer, next_ptr, max_len);
				next_ptr = next_ptr + max_len;
			} else {
				strncpy(trim_buffer, next_ptr, max_len);
				next_ptr = NULL;
			}

			/* Send to servers */
			if (ircd_sprintf
			    (buffer, BUF_SIZE, "PRIVMSG %s :%s",
			     cmd->params[0], trim_buffer)) {
				return 0;
			}
			servers_relay(s_ptr, sender->nick, buffer);
			/* Send to local clients */
			client_msg_by_server_mask(sender,
						  &(cmd->params[0][1]),
						  "PRIVMSG", cmd->text,
						  NULL);
		}
		return 1;
	}

	/* Send using hostmasks */
	if (cmd->params[0][0] == '#' && sender->mode & MODE_IRCOP) {
		size_t max_len = 0;
		char *next_ptr = NULL;
		char trim_buffer[BUF_SIZE];

		max_len = BUF_SIZE - 14 - strlen(recip->nick);
		next_ptr = cmd->text;

		while (next_ptr) {
			if (strlen(next_ptr) > max_len) {
				strncpy(trim_buffer, next_ptr, max_len);
				next_ptr = next_ptr + max_len;
			} else {
				strncpy(trim_buffer, next_ptr, max_len);
				next_ptr = NULL;
			}

			/* Send to servers */
			if (ircd_sprintf
			    (buffer, BUF_SIZE, "PRIVMSG %s :%s",
			     cmd->params[0], trim_buffer)) {
				/* Overflow */
				return 0;
			}
			servers_relay(s_ptr, sender->nick, buffer);
			/* Local Clients */
			client_msg_by_host_mask(sender,
						&(cmd->params[0][1]),
						"PRIVMSG", cmd->text,
						NULL);
		}
		return 1;
	}

	/* Send error if its a local client */
	if (c_ptr) {
		ircd_sprintf(buffer, BUF_SIZE, "%s :No such nick/channel",
			     cmd->params[0]);
		send_numeric(sender, get_my_server(), 401, buffer);
	}
	return 1;

}

/* 
 * send a privmsg to a user 
 */
static int privmsg_to_user(ClientInfo sender, ClientInfo recip, char *text)
{

	/* form end of string */
	if (text == NULL) {
		return 0;
	} else {
		char trim_buffer[BUF_SIZE];
		char cmd_buff[BUF_SIZE];
		size_t max_len = 0;
		char *next_ptr = NULL;

		if ((recip->mode & MODE_AWAY)
		    && (recip->server == get_my_server())) {
			ircd_sprintf(cmd_buff, BUF_SIZE, "%s :%s",
				     recip->nick, recip->away_msg);
			send_numeric(sender, get_my_server(), 301,
				     cmd_buff);
		}
		max_len = BUF_SIZE - 14 - strlen(recip->nick);
		next_ptr = text;

		while (next_ptr) {
			if (strlen(next_ptr) > max_len) {
				strncpy(trim_buffer, next_ptr, max_len);
				next_ptr = next_ptr + max_len;
			} else {
				strncpy(trim_buffer, next_ptr, max_len);
				next_ptr = NULL;
			}

			if (ircd_sprintf
			    (cmd_buff, BUF_SIZE, "PRIVMSG %s :%s",
			     recip->nick, trim_buffer)) {
				return 0;
			} else {
				send_client_client(recip, sender,
						   cmd_buff);
			}
		}

	}

	return 1;
}

/* 
 * send a privmsg to a channel 
 */
static int privmsg_to_channel(ServerInfo s_ptr,
			      ClientInfo sender,
			      ChannelInfo chan, char *text)
{
	char buffer[BUF_SIZE];
	char trim_buffer[BUF_SIZE];
	char *next_ptr = NULL;
	ChannelMember curr_client = NULL;
	int rand_no;
	size_t max_len = 0;

	if (!channel_msg_check_perms(chan, sender)) {
		/* Tell them they don't have permission */
		ircd_sprintf(buffer, BUF_SIZE,
			     "%s :Cannot send to channel", chan->name);
		send_numeric(sender, get_my_server(), 404, buffer);
		return 1;
	}

	/* Keep an eye open for buffer overflows */
	max_len = BUF_SIZE - 14 - strlen(chan->name);
	next_ptr = text;

	while (next_ptr) {
		if (strlen(next_ptr) > max_len) {
			strncpy(trim_buffer, next_ptr, max_len);
			next_ptr = next_ptr + max_len;
		} else {
			strncpy(trim_buffer, next_ptr, max_len);
			next_ptr = NULL;
		}

		/* Assemble output buffer */
		if (ircd_sprintf
		    (buffer, BUF_SIZE, "PRIVMSG %s :%s", chan->name,
		     trim_buffer)) {
			return 0;
		} else {
			send_to_channel_selective(sender, chan, s_ptr,
						  buffer);
		}
	}

	return 1;
}
