/* This file is part of Chunky Monkey IRCD 
 *
 * Chunky Monkey IRCD is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Chunky Monkey IRCD is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Chunky Monkey IRCD; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * $Id: cmd_quit.c,v 1.12.4.10 2002/05/24 21:50:23 lloydy Exp $
 *
 */

#include "modules.h"

CMIRCD_MODULE(1.02, CMD_CLIENT | CMD_SERVER);

/* 
* cmd_quit - deals with quit commands 
* Just Moved out of the core IRCD
*/
int cmd_quit(ClientInfo c_ptr, ServerInfo s_ptr, Cmd cmd)
{
	ClientInfo origin = NULL;
	ChannelMember curr_user = NULL;
	int rand_no;
	int i;
	char buffer[BUF_SIZE];


	/* Get a client pointer, if the message is from a server */
	if (!c_ptr) {
		/* try and locate client */
		origin = client_locate(cmd->origin);
	} else {
		origin = c_ptr;
	}

	if (!origin) {

		return 0;
	}

	/* Setup buffer */
	ircd_sprintf(buffer, BUF_SIZE, "QUIT :%s",
		     cmd->text ? cmd->text : "");
	rand_no = rand() % 1000 + 1;

	for (i = 0;
	     i < origin->chanlist_no && origin->chan_list[i] != NULL;
	     i++) {

		/* remove from channel user list */
		channel_rem_client(origin, origin->chan_list[i]);

		/* Inform the channel */
		curr_user = origin->chan_list[i]->users_start;
		while (curr_user) {
			if (curr_user->user_info->rand_no != rand_no
			    && curr_user->user_info->fd != -1) {
				send_client_client(curr_user->user_info,
						   origin, buffer);
				curr_user->user_info->rand_no = rand_no;
			}
			curr_user = curr_user->next;
		}

		/* Close channel if needed */
		channel_needclose(origin->chan_list[i]);

		/* Set pointer to NULL */
		origin->chan_list[i] = NULL;
	};

	/* Return Error message to client */
	/* Don't think its needed */
	/* sprintf(buffer,"ERROR :Have fun\n"); */
	/* send_raw(sender->fd,buffer); */

	

	/* Do some counter reduction */
	if (origin->mode & MODE_IRCOP) {
		server_state.ops--;
	}
	origin->server->client_count--;

	/* Relay to other servers */
	servers_relay(s_ptr, origin->nick, buffer);
#ifdef USE_OPENSSL
	if(origin->ssl) {
		SSL_shutdown(origin->ssl);
	}
#endif

	/* Remove from tree */
	if (!client_remove(origin->nick)) {
		printf("Failed to remove client in %s:%d\n", __FILE__,
		       __LINE__);
	}

	/* If we are highest, find next highest */
	if (origin->fd != -1) {

		ircd_fdlist_clear_entry(origin->fd);

		/* Close fd */
		close(origin->fd);
	}

	return 1;
}
